﻿// NaverQuery.cs
//

using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using System.Text;

namespace LinqProviders.LinqToNaver {

    internal sealed class NaverQuery<T> : INaverQueryable<T> {

        private NaverQueryProvider _queryProvider;
        private Expression _expression;

        public NaverQuery(NaverQueryProvider queryProvider) {
            _queryProvider = queryProvider;
            _expression = Expression.Constant(this);
        }

        public NaverQuery(NaverQueryProvider queryProvider, Expression expression) {
            _queryProvider = queryProvider;
            _expression = expression;
        }

        #region IEnumerable Members
        IEnumerator IEnumerable.GetEnumerator() {
            return ((IEnumerable<T>)this).GetEnumerator();
        }
        #endregion

        #region IEnumerable<T> Members
        IEnumerator<T> IEnumerable<T>.GetEnumerator() {
            IEnumerable<T> sequence = _queryProvider.Execute<IEnumerable<T>>(_expression);
            return sequence.GetEnumerator();
        }
        #endregion

        #region IQueryable Members
        Type IQueryable.ElementType {
            get {
                return typeof(T);
            }
        }

        Expression IQueryable.Expression {
            get {
                return _expression;
            }
        }

        IQueryProvider IQueryable.Provider {
            get {
                return _queryProvider;
            }
        }
        #endregion
    }
}
