﻿// NaverQueryable.cs
//

using System;
using System.Linq;
using System.Linq.Expressions;
using System.Reflection;

namespace LinqProviders.LinqToNaver {

    public static class NaverQueryable {

        public static INaverQueryable<T> SafeResults<T>(this INaverQueryable<T> source) {
            MethodInfo safeResultsMethod = ((MethodInfo)MethodBase.GetCurrentMethod()).MakeGenericMethod(new Type[] { typeof(T) });
            Expression safeResultsExpression = Expression.Call(safeResultsMethod, source.Expression);

            return (INaverQueryable<T>)source.Provider.CreateQuery<T>(safeResultsExpression);
        }

        public static INaverQueryable<T> LocalResults<T>(this INaverQueryable<T> source, string location) {
            MethodInfo localResultsMethod = ((MethodInfo)MethodBase.GetCurrentMethod()).MakeGenericMethod(new Type[] { typeof(T) });
            Expression localResultsExpression =
                Expression.Call(localResultsMethod,
                                source.Expression, Expression.Constant(location));

            return (INaverQueryable<T>)source.Provider.CreateQuery<T>(localResultsExpression);
        }

        public static INaverQueryable<T> ScopeResults<T>(this INaverQueryable<T> source, string domain) {
            MethodInfo localResultsMethod = ((MethodInfo)MethodBase.GetCurrentMethod()).MakeGenericMethod(new Type[] { typeof(T) });
            Expression localResultsExpression =
                Expression.Call(localResultsMethod,
                                source.Expression, Expression.Constant(domain));

            return (INaverQueryable<T>)source.Provider.CreateQuery<T>(localResultsExpression);
        }
    }
}
