﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml.Linq;
using LinqProviders.Utility;

namespace LinqProviders.LinqToNaver
{
    public class BlogSearchResult : NaverSearchResult
    {

        /// <summary>
        /// Gets or sets the sort. The sort. date : 날짜순(기본값), sim : 유사도순
        /// </summary>
        /// <value>The sort. date : 날짜순(기본값), sim : 유사도순</value>
        public string Sort { get; set; }

        public string Rss { get; set; }
        public string CnTitle { get; set; }
        public Uri CnLink { get; set; }
        public String CnDescription { get; set; }
        public DateTime LastBuildDate { get; set; }
        
        public string Title { get; set; }
        public Uri Link { get; set; }
        public String Description { get; set; }
        public string BloggerName { get; set; }
        public string BloggerLink { get; set; }

        public static IEnumerable<BlogSearchResult> Parse(string  input)
        {
            if (string.IsNullOrEmpty(input))
                return null;

            XDocument xdoc = XDocument.Parse(input);
            var chan = xdoc.Descendants("channel").FirstOrDefault();
            if (chan == null)
                return null;

            Channel rootChannel = new Channel();
            rootChannel.Title = chan.Element("title")!=null ?  chan.Element("title").Value : string.Empty ;
            rootChannel.Link = ConversionHelpers.ToUri(chan.Element("link")!=null ?  chan.Element("link").Value : "http://" );
            rootChannel.Description = chan.Element("description") != null ? chan.Element("description").Value : "";
            rootChannel.Total = int.Parse(chan.Element("total") != null ? chan.Element("total").Value : "0");
            rootChannel.Display = int.Parse(chan.Element("display") != null ? chan.Element("display").Value : "0");
            rootChannel.Start = int.Parse(chan.Element("start") != null ? chan.Element("start").Value : "0");
            rootChannel.LastBuildDate = DateTime.Parse(chan.Element("lastBuildDate") != null ? chan.Element("lastBuildDate").Value : DateTime.Now.ToString());

            return from items in chan.Descendants("item")
                   select new BlogSearchResult
                   {
                       CnTitle = rootChannel.Title,
                       CnLink = rootChannel.Link,
                       CnDescription = rootChannel.Description,
                       LastBuildDate = rootChannel.LastBuildDate,
                       Total = rootChannel.Total,
                       Display = rootChannel.Display,
                       Start = rootChannel.Start,
                       Title = items.Element("title") != null ? items.Element("title").Value : "",
                       Link = ConversionHelpers.ToUri(items.Element("link") != null ? items.Element("link").Value : "http://"),
                       Description = items.Element("description") != null ? items.Element("description").Value : "",
                       BloggerName = items.Element("bloggername") != null ? items.Element("bloggername").Value : string.Empty,
                       //BloggerLink = ConversionHelpers.ToUri(items.Element("bloggerlink") != null ? items.Element("bloggerlink").Value : "http://", UriKind.Absolute),
                       BloggerLink = items.Element("bloggerlink") != null ? items.Element("bloggerlink").Value : "http://"
                   };
        }

        protected override object Parse(string input, string requestedQuery, out int count, out int totalCount)
        {
            IEnumerable<BlogSearchResult> resultList = new List<BlogSearchResult>();
            count = 0;
            totalCount = 0;

            if (String.IsNullOrEmpty(input) == false)
            {
                resultList = BlogSearchResult.Parse(input);
            }

            totalCount = resultList.FirstOrDefault() != null ? resultList.FirstOrDefault().Total : 0;

            return resultList;
        }
    }

    public class BlogItem
    {
        public string Title { get; set; }
        public Uri Link { get; set; }
        public String Description { get; set; }
        public string BloggerName { get; set; }
        public Uri BloggerLink { get; set; }
    }
}
