﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Xml.Linq;
using LinqProviders.Utility;

namespace LinqProviders.LinqToNaver
{
    /// <summary>
    /// 이미지 검색
    /// </summary>
    public class ImageSearchResult : NaverSearchResult
    {
        public string Rss { get; set; }
        public string CnTitle { get; set; }
        public Uri CnLink { get; set; }
        public String CnDescription { get; set; }
        public DateTime LastBuildDate { get; set; }


        /// <summary>
        /// Gets or sets the sort.
        /// </summary>
        /// <value>The sort.sim : 유사도순 (기본값), date : 날짜순</value>
        public String Sort { get; set; }

        /// <summary>
        /// Gets or sets the filter.
        /// </summary>
        /// <value>The filter. all : 전체 이미지(기본값), large : 큰 이미지만 제공, medium : 중간 이미지만 제공 , small : 작은 이미지만 제공</value>
        public String Filter { get; set; }


        public string Title { get; set; }
        public Uri Link { get; set; }
        public Uri Thumbnail { get; set; }
        public int SizeHeight { get; set; }
        public int SizeWidth { get; set; }

        /// <summary>
        /// Gets the channel query.
        /// </summary>
        /// <param name="xdoc">The xdoc.</param>
        /// <returns></returns>
        private IEnumerable<ImageSearchResult> Parse(string input)
        {
            if (string.IsNullOrEmpty(input))
                return null;

            XDocument xdoc = XDocument.Parse(input);
            var chan = xdoc.Descendants("channel").FirstOrDefault();
            if (chan == null)
                return null;


            Channel rootChannel = new Channel();
            rootChannel.Title = chan.Element("title")!=null ?  chan.Element("title").Value : string.Empty ;
            rootChannel.Link = ConversionHelpers.ToUri(chan.Element("link")!=null ?  chan.Element("link").Value : "http://" );
            rootChannel.Description = chan.Element("description") != null ? chan.Element("description").Value : "";
            rootChannel.Total = int.Parse(chan.Element("total") != null ? chan.Element("total").Value : "0");
            rootChannel.Display = int.Parse(chan.Element("display") != null ? chan.Element("display").Value : "0");
            rootChannel.Start = int.Parse(chan.Element("start") != null ? chan.Element("start").Value : "0");
            rootChannel.LastBuildDate = DateTime.Parse(chan.Element("lastBuildDate") != null ? chan.Element("lastBuildDate").Value : DateTime.Now.ToString());

            return from items in chan.Descendants("item")
                   select new ImageSearchResult
                   {
                       CnTitle = rootChannel.Title,
                       CnLink = rootChannel.Link,
                       CnDescription = rootChannel.Description,
                       LastBuildDate = rootChannel.LastBuildDate,
                       Total = rootChannel.Total,
                       Display = rootChannel.Display,
                       Start = rootChannel.Start,
                       Title = items.Element("title") != null ? items.Element("title").Value : "",
                       Link = ConversionHelpers.ToUri(items.Element("link") != null ? items.Element("link").Value : "http://"),
                       SizeHeight = int.Parse(items.Element("sizeheight") != null ? items.Element("sizeheight").Value : "0"),
                       SizeWidth = int.Parse(items.Element("sizewidth") != null ? items.Element("sizewidth").Value : "0"),
                       Thumbnail = ConversionHelpers.ToUri(items.Element("thumbnail") != null ? items.Element("thumbnail").Value : "http://", UriKind.Absolute)
                   };
        }

        protected override object Parse(string input, string requestedQuery, out int count, out int totalCount)
        {
            IEnumerable<ImageSearchResult> resultList = new List<ImageSearchResult>();
            count = 0;
            totalCount = 0;

            if (String.IsNullOrEmpty(input) == false)
            {
                resultList = Parse(input);
            }
            totalCount = resultList.FirstOrDefault() != null ? resultList.FirstOrDefault().Total : 0;
            return resultList;
        }
    }

}
