﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml.Linq;
using LinqProviders.Utility;

namespace LinqProviders.LinqToNaver
{
    public class PageSearchResult : NaverSearchResult
    {
        public string Rss { get; set; }
        public string CnTitle { get; set; }
        public Uri CnLink { get; set; }
        public String CnDescription { get; set; }
        public DateTime LastBuildDate { get; set; }

        public string Title { get; set; }
        public Uri Link { get; set; }
        public String Description { get; set; }
        public string Domain { get; set; }

        private IEnumerable<PageSearchResult> Parse(string input)
        {
            if (string.IsNullOrEmpty(input))
                return null;

            XDocument xdoc = XDocument.Parse(input);
            var chan = xdoc.Descendants("channel").FirstOrDefault();
            if (chan == null)
                return null;


            Channel rootChannel = new Channel();
            rootChannel.Title = chan.Element("title") != null ? chan.Element("title").Value : string.Empty;
            rootChannel.Link = ConversionHelpers.ToUri(chan.Element("link") != null ? chan.Element("link").Value : "http://");
            rootChannel.Description = chan.Element("description") != null ? chan.Element("description").Value : "";
            rootChannel.Total = int.Parse(chan.Element("total") != null ? chan.Element("total").Value : "0");
            rootChannel.Display = int.Parse(chan.Element("display") != null ? chan.Element("display").Value : "0");
            rootChannel.Start = int.Parse(chan.Element("start") != null ? chan.Element("start").Value : "0");
            rootChannel.LastBuildDate = DateTime.Parse(chan.Element("lastBuildDate") != null ? chan.Element("lastBuildDate").Value : DateTime.Now.ToString());

            return from items in chan.Descendants("item")
                   select new PageSearchResult
                   {
                       CnTitle = rootChannel.Title,
                       CnLink = rootChannel.Link,
                       CnDescription = rootChannel.Description,
                       LastBuildDate = rootChannel.LastBuildDate,
                       Total = rootChannel.Total,
                       Display = rootChannel.Display,
                       Start = rootChannel.Start,
                       Title = items.Element("title") != null ? items.Element("title").Value : "",
                       Link = ConversionHelpers.ToUri(items.Element("link") != null ? items.Element("link").Value : "http://"),
                       Description = items.Element("description") != null ? items.Element("description").Value : "",
                   };
        }

        protected override object Parse(string input, string requestedQuery, out int count, out int totalCount)
        {
            IEnumerable<PageSearchResult> resultList = new List<PageSearchResult>();
            count = 0;
            totalCount = 0;

            if (String.IsNullOrEmpty(input) == false)
            {
                resultList = Parse(input);
            }
            totalCount = resultList.FirstOrDefault() != null ? resultList.FirstOrDefault().Total : 0;
            return resultList;
        }
    }

}
