﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Xml.Linq;
using LinqProviders.Utility;

namespace LinqProviders.LinqToNaver
{
    public class ShopSearchResult : NaverSearchResult
    {
        public string Rss { get; private set; }
        public string CnTitle { get; private set; }
        public Uri CnLink { get; private set; }
        public String CnDescription { get; private set; }
        public DateTime LastBuildDate { get; private set; }


        /// <summary>
        /// Gets or sets the sort.
        /// </summary>
        /// <value>The sort. 정렬옵션입니다, sim : 유사도순 (기본값), date : 날짜순, asc : 가격오름차순, dsc : 가격내림차순</value>
        public string Sort { get; set; }


        public string Title { get; private set; }
        /// <summary>
        /// Gets or sets the link.검색 결과 문서의 하이퍼텍스트 link를 나타냅니다.
        /// </summary>
        /// <value>The link.</value>
        public Uri Link { get; private set; }
        /// <summary>
        /// Gets or sets the image.썸네일 이미지의 URL입니다. 이미지가 있는 경우만 나타납니다.
        /// </summary>
        /// <value>The image.</value>
        public string Image { get; private set; }
        /// <summary>
        /// Gets or sets the high price.
        /// 최고가 정보입니다. 최고가 정보가 없거나 가격비교 데이터가 없는 경우 0으로 표시됩니다.
        /// </summary>
        /// <value>The high price.</value>
        public string HighPrice { get; private set; }
        /// <summary>
        /// Gets or sets the low price.
        /// 최저가 정보입니다. 최저가 정보가 없는 경우 0으로 표시되며, 가격비교 데이터가 없는 경우 이 필드는 가격을 나타냅니다. 
        /// </summary>
        /// <value>The low price.</value>
        public string LowPrice { get; private set; }

        /// <summary>
        /// Gets the channel query.
        /// </summary>
        /// <param name="xdoc">The xdoc.</param>
        /// <returns></returns>
        private IEnumerable<ShopSearchResult> Parse(string input)
        {
            if (string.IsNullOrEmpty(input))
                return null;

            XDocument xdoc = XDocument.Parse(input);
            var chan = xdoc.Descendants("channel").FirstOrDefault();
            if (chan == null)
                return null;

            Channel rootChannel = new Channel();
            rootChannel.Title = chan.Element("title")!=null ?  chan.Element("title").Value : string.Empty ;
            rootChannel.Link = ConversionHelpers.ToUri(chan.Element("link")!=null ?  chan.Element("link").Value : "http://" );
            rootChannel.Description = chan.Element("description") != null ? chan.Element("description").Value : "";
            rootChannel.Total = int.Parse(chan.Element("total") != null ? chan.Element("total").Value : "0");
            rootChannel.Display = int.Parse(chan.Element("display") != null ? chan.Element("display").Value : "0");
            rootChannel.Start = int.Parse(chan.Element("start") != null ? chan.Element("start").Value : "0");
            rootChannel.LastBuildDate = DateTime.Parse(chan.Element("lastBuildDate") != null ? chan.Element("lastBuildDate").Value : DateTime.Now.ToString());

            return from items in chan.Descendants("item")
                   select new ShopSearchResult
                   {
                       CnTitle = rootChannel.Title,
                       CnLink = rootChannel.Link,
                       CnDescription = rootChannel.Description,
                       LastBuildDate = rootChannel.LastBuildDate,
                       Total = rootChannel.Total,
                       Display = rootChannel.Display,
                       Start = rootChannel.Start,
                       Title = items.Element("title") != null ? items.Element("title").Value : "",
                       Link = ConversionHelpers.ToUri(items.Element("link") != null ? items.Element("link").Value : "http://"),
                       Image = items.Element("image") != null ? items.Element("image").Value : string.Empty,
                       LowPrice = items.Element("lprice") != null ? items.Element("lprice").Value : "0",
                       HighPrice = items.Element("hprice") != null ? items.Element("hprice").Value : "0"
                   };
        }

        protected override object Parse(string input, string requestedQuery, out int count, out int totalCount)
        {
            IEnumerable<ShopSearchResult> resultList = new List<ShopSearchResult>();
            count = 0;
            totalCount = 0;

            if (String.IsNullOrEmpty(input) == false)
            {
                resultList = Parse(input);
            }

            totalCount = resultList.FirstOrDefault() != null ? resultList.FirstOrDefault().Total : 0;

            return resultList;
        }
    }

}
